% Script to reproduce Fig. 5 (bottom pannel) of:
% de Cheveign, A., Arzounian, D., Robust detrending, rereferencing,
% outlier detection, and inpainting for multichannel data,(submitted to
% NeuroImage).
%
% Note: this script requires the NoiseTools toolbox
% (http://audition.ens.fr/adc/NoiseTools/). It should work with version
% 05-Jul-2017.

clear

%% Create artificial, corrupted sinusoidal signal:
N=10000; % Number of samples
sr=10000; % Sampling rate (Hz)
t=(0:N-1)'/sr; % Time vector
% The noise is a 50-Hz sinusoid:
noise=sin(2*pi*50*t);
% The clean signal is a 1.25-Hz sinusoid
signal=-sin(2.5*pi*t);
% The signal is a weighted sum of clean signal and noise:
x=4*signal+noise;
% Add short glitch:
x(1001:1100)=200;

%% Plot raw signal and detrended versions:

% Prepare figure:
figure(2); clf; set(gcf,'color', [1 1 1], 'position',[100 100 600 213])

% Plot raw signal on the left:
axes('position', [12,20,25,70]/100)
plot(t,x, 'k');
hold on; plot(t,0*t,':k') % Add zero baseline
% Set axes properties:
set(gca,'fontsize',14,'ytick', [-10 0 10], 'xtick', [0 1]); title('raw'); xlabel('time (s)'); ylabel('A.U.')
ylim([-10 10]);

% Plot result of standard (i.e. non-robust) detrending in the middle:
axes('position', [42,20,25,70]/100)
% Detrending consists here in removing a 50-Hz sinusoidal trend. We
% therefore use the noise signal as the function basis. Non-robust
% detrending is obtained using a high threshold for outliers, e.g. 1000
% here:
plot(t,nt_detrend(x,[],[],noise,1000), 'k');
% Set axes properties:
hold on; plot(t,0*t,':k')
set(gca,'fontsize',14,'ytick',[], 'xtick', [0 1]); title('detrend'); xlabel('time (s)'); 
ylim([-10 10]);

% Plot result of robust detrending on the right:
axes('position', [72,20,25,70]/100)
% Same detrending as previouly, but using a lower threshold, e.g. 3 here:
plot(t,nt_detrend(x,[],[],noise,3), 'k');
% Set axes properties:
hold on; plot(t,0*t,':k')
set(gca,'fontsize',14,'ytick',[], 'xtick', [0 1]); title('robust detrend'); xlabel('time (s)'); 
ylim([-10 10]);

% Save 3 panels:
set(gcf, 'PaperPositionMode', 'auto');
print ('-depsc2', 'Fig_5_bottom')

